//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Text;
namespace Testing
{
	class ArrayMain
	{
		static void Single()
		{
			// Declare an array 
			int [] a = new int[10];
			for(int i = 0; i < a.Length; i++)
				a[i] = i;
			// Initialize and declare in one statement
			int [] b = new int [] {0, 1, 2 ,3 ,4 ,5, 6, 7, 8, 9};
			Console.WriteLine("{0} {1} {2}", a.Equals(b), a.Length, b.Length);
			// Copy two elements from 'a' starting at index 5 to 
			// 'b' starting at 0.
			Array.Copy(a, 5, b, 0, 2);
			foreach(int i in b)
				Console.WriteLine("{0}", i);
			// Create an array of strings
			string [] c = new string [] {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};
			// Iterate through the array of strings
			foreach(string s in c)
				Console.WriteLine("{0}", s);
		}
		static void Multiple()
		{
			// Declare a multi-dimensional array
			int [,] a = new int[5,5];
			for(int i = 0; i < a.GetLength(0); i++)
				for(int j = 0; j < a.GetLength(1); j++)
					a[i,j] = i * a.GetLength(1) + j;
			// Initialize and declare in one statement
			int [,] b = new int [,] {{0, 0, 0, 0, 0},
									 {1, 1, 1, 1, 1},
									 {2, 2, 2, 2, 2},
									 {3, 3, 3, 3, 3},
									 {4, 4, 4, 4, 4}};
			Console.WriteLine("{0} {1} dimensions {2}x{3} {4}x{5}", a.Equals(b),
				                                                    a.Rank,
				                                                    a.GetLength(0),
				                                                    a.GetLength(1),
				                                                    b.GetLength(0),
				                                                    b.GetLength(1));
			// When copying between multidimensional arrays, the array
			// behaves like a long one-dimensional array, where the rows
			// (or columns) are conceptually laid end to end. For example,
			// if an array has three rows (or columns) with four elements
			// each, copying six elements from the beginning of the array
			// would copy all four elements of the first row (or column)
			// and the first two elements of the second row (or column).
			Array.Copy(a, 5, b, 5, 5);
			for(int i = 0; i < a.GetLength(0); i++)
			{
				for(int j = 0; j < a.GetLength(1); j++)
				{
					Console.Write("{0}", b[i,j]);
				}
				Console.WriteLine();
			}
			// Create a two-dimensional array of strings
			string [,] c = new string [,]{{"0 - Monday",
										   "0 - Tuesday",
										   "0 - Wednesday",
										   "0 - Thursday",
										   "0 - Friday",
										   "0 - Saturday",
										   "0 - Sunday"},
										  {"1 - Monday",
										   "1 - Tuesday",
										   "1 - Wednesday",
										   "1 - Thursday",
										   "1 - Friday",
										   "1 - Saturday",
										   "1 - Sunday"}};
			// Iterate through the array
			foreach(string s in c)
					Console.WriteLine(s);
			int [,,,] d = new int [5,5,5,5];
			Console.WriteLine("{0} dimensions", d.Rank);
		}
		static void Jagged()
		{
			// Declare a multi-dimensional array
			int [][] a = new int[5][];
			for(int i = 0; i < a.Length; i++)
				a[i] = new int [i + 1];
			for(int i = 0; i < a.Length; i++)
			{
				for(int j = 0; j < a[i].Length; j++)
				{
					a[i][j] = i;
				}
			}
			foreach(int [] ia in a)
			{
				foreach(int i in ia)
					Console.Write("{0}", i);
				Console.WriteLine();
			}
			// Initialize and declare in one statement
			int [][] b = {new int [] {0},
						  new int [] {1, 1},
						  new int [] {2, 2, 2},
						  new int [] {3, 3, 3, 3},
						  new int [] {4, 4, 4, 4, 4}};
			foreach(int [] ia in b)
			{
				foreach(int i in ia)
					Console.Write("{0}", i);
				Console.WriteLine();
			}
		}
		static void Main(string[] args)
		{
			Single();
			Multiple();
			Jagged();
		}
	}
}
