//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Collections;

namespace Testing
{
	// Declare the collection:
	public class CSharpCollection 
	{
		int[] items;

		public CSharpCollection() 
		{
			items = new int[5] {1, 2, 3, 4, 5};
		}

		public CSharpEnumerator GetEnumerator() 
		{
			return new CSharpEnumerator(this);
		}

		// Declare the enumerator class:
		public class CSharpEnumerator 
		{
			int nIndex;
			CSharpCollection collection;
			public CSharpEnumerator(CSharpCollection coll) 
			{
				collection = coll;
				nIndex = -1;
			}

			public bool MoveNext() 
			{
				nIndex++;
				return(nIndex < collection.items.GetLength(0));
			}

			public int Current 
			{
				get 
				{
					return(collection.items[nIndex]);
				}
			}
		}
	}

	public class GenericCollection: IEnumerable 
	{
		int[] items;
		public GenericCollection() 
		{
			items = new int[5] {6, 7, 8, 9, 10};
		}

		public GenericEnumerator GetEnumerator() 
		{
			return new GenericEnumerator(this);
		}

		// Implement the GetEnumerator() method:
		IEnumerator IEnumerable.GetEnumerator() 
		{
			return GetEnumerator();
		}

		// Declare the enumerator and implement the IEnumerator interface:
		public class GenericEnumerator: IEnumerator 
		{
			int nIndex;
			GenericCollection collection;
			public GenericEnumerator(GenericCollection coll) 
			{
				collection = coll;
				nIndex = -1;
			}

			public void Reset() 
			{
				nIndex = -1;
			}

			public bool MoveNext() 
			{
				nIndex++;
				return(nIndex < collection.items.GetLength(0));
			}

			public int Current 
			{
				get 
				{
					return(collection.items[nIndex]);
				}
			}

			// The current property on the IEnumerator interface:
			object IEnumerator.Current 
			{
				get 
				{
					return(Current);
				}
			}
		}
	}

	class ForeachMain
	{
		static void Main(string[] args)
		{
			string [] sa = new string [] { "This", "is", "a", "test" };
			foreach(string s in sa)
			{
				Console.WriteLine(s);
			}
			CSharpCollection cscol = new CSharpCollection();
			Console.WriteLine("Values in the C# collection are:");
			foreach (int i in cscol) 
			{
				Console.WriteLine(i);
			}
			GenericCollection gencol = new GenericCollection();
			Console.WriteLine("Values in the generic collection are:");
			foreach (int i in gencol) 
			{
				Console.WriteLine(i);
			}
		}
	}
}
