//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;

namespace Testing
{
	class Shape
	{
		public virtual void Draw()
		{
			Console.WriteLine("Drawing a Shape");
		}
	}
	
	class Circle: Shape
	{
		double x;
		double y;
		double r;
		public Circle(double x, double y, double r)
		{
			this.x = x;
			this.y = y;
			this.r = r;
		}
		override public void Draw()
		{
			Console.WriteLine("Drawing a circle of radius {0}", r);
		}
	}
	
	class Triangle: Shape
	{
		double [,] vertices;
		public Triangle(double x1, double y1,
		                double x2, double y2,
		                double x3, double y3)
		{
			vertices = new double [,] {{x1, y1},
			                           {x2, y2},
			                           {x3, y3}};
		}
		override public void Draw()
		{
			Console.WriteLine("Drawing a triangle");
		}
	}
	
	class Square: Shape
	{
		double x;
		double y;
		double s;
		public Square(double x, double y, double s)
		{
			this.x = x;
			this.y = y;
			this.s = s;
		}
		override public void Draw()
		{
			Console.WriteLine("Drawing a {0}x{0} square", s);
		}
	}
	
	class PolymorphicMain
	{
		static void Main(string[] args)
		{
			Shape [] shapes = new Shape [] {new Circle(0,0,2),
											new Triangle(0,0,0,1,1,1),
											new Square(0,0,5)};
			foreach(Shape s in shapes)
			{
				s.Draw();
			}
		}
	}
}
